@@ -980,7 +980,7 @@ bus_activation_send_pending_auto_activation_messages (BusActivation  *activation
         {
           DBusConnection *addressed_recipient;
           
-          addressed_recipient = bus_service_get_primary_owner (service);
+          addressed_recipient = bus_service_get_primary_owners_connection (service);
 
           /* Check the security policy, which has the side-effect of adding an
            * expected pending reply.
@@ -1886,13 +1886,15 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
   if (!init_service_reload_test (&directory))
     _dbus_assert_not_reached ("could not initiate service reload test");
  
-  if (!do_service_reload_test (&directory, FALSE));
+  if (!do_service_reload_test (&directory, FALSE))
+    ; /* Do nothing? */
   
   /* Do OOM tests */
   if (!init_service_reload_test (&directory))
     _dbus_assert_not_reached ("could not initiate service reload test");
  
-  if (!do_service_reload_test (&directory, TRUE));
+  if (!do_service_reload_test (&directory, TRUE))
+    ; /* Do nothing? */
  
   /* Cleanup test directory */
   if (!cleanup_service_reload_test (&directory))
